
#include <Array.au3>

#cs

ID Populator v0.1
Samuel Murray, May 2015

SDLXLIFF files don't store formatting tags' actual content inside the segmented text, but instead refers to it by an ID that is defined in the head of the SDLXLIFF file.  This means that programs like Wordfast Pro or OmegaT that read SDLXLIFF files don't show the user what a tag means (it shows only the tag ID number).

The ID Populator scripts alter the SDXLIFF file so that tags' meanings are present inside the segments, so that if users process those SDXLIFF files in another tool, he/she may know what the tags actually mean.  The scripts give the option to clarify all tags, or only tags that the user selects (e.g. if the user is only interested in seeing which tags mean "highlight yellow", then he can set it so that only those tags are explained in the segments).

Script #1 creates a tag list file with all formatting tags in it.
The user then edits that list file to (a) remove all lines that he is not interested in or (b) mark up only the formatting that he is interested in.

Script #2 uses the tag list file to edit the SDLXLIFF file by adding the formatting to the segments, in curly brackets.

Script #3 is used in the end, to remove the added formatting information from the SDLXLIFF file again.

The tag list file has 3 columns, namely tag ID, tag content, and the segment in which it occurs.  Column 3 is not required to be present when running script #2.  Any line with fewer than 2 columns is not processed.

At the top of the tag list file are two YES/NO questions with two columns each:
* Question 1: Process all lines?
If the word "YES" is in the second column, then all lines in the tag list file is processed and the full text in column 2 of each tag is inserted into the SDLXLIFF file; if the word "NO" is in the second column, then only lines that contain {{ is processed, and only the text between {{ and }} is inserted into the SDLXLIFF file.
* Question 2: Put it inside G tag?
If "YES", then the script will change e.g. <g id="123"> to <g ttx="{{tag content}}" id="123">, which will cause Trados to complain when it loads the file, but it will load it anyway, and when you press Save in Trados, it quietly removes the inserted content.  If "NO", then the tag content is inserted into the SDXLIFF file directly after the relevant G tag, between {{ and }}.  THis means that script #3 has to be used to remove it afterwards, otherwise it remains visible inside Trados and insert the final file also.

#ce

Global $segments[1]
Global $tagcontents[1]
Global $tagnums[1]

; 1. Add segments to an array.

$sdlfileread = FileRead (FileOpen (FileOpenDialog ("Select SDLXLIFF file", @ScriptDir, "SDXLIFF (*.sdlxliff)|All (*.*)"), 256)) ; 256 = UTF8 without BOM

$sdlfile1stsplit = StringSplit ($sdlfileread, "<trans-unit", 1)

For $i = 2 to $sdlfile1stsplit[0]
 
$sdlfile2ndsplit = StringSplit ($sdlfile1stsplit[$i], "</trans-unit>", 1)

_ArrayAdd ($segments, $sdlfile2ndsplit[1])

Next

$numsegmentsx = Ubound($segments)
$numsegments = $numsegmentsx - 1

; ==

; 2. Add tag content to an array.

$sdlheadsplit = StringSplit ($sdlfileread, "</header>", 1)

$sdlfileAsplit = StringSplit ($sdlheadsplit[1], '<tag id="', 1)

For $j = 2 to $sdlfileAsplit[0]

$tagnumAsplit = StringSplit ($sdlfileAsplit[$j], '">', 1)
$tagnum = $tagnumAsplit[1]

_ArrayAdd ($tagnums, $tagnum)

$sdlfileBsplit = StringSplit ($sdlfileAsplit[$j], '</tag>', 1)

$tagcontentraw = StringReplace ($sdlfileBsplit[1], $tagnum & '">', "", 1)

; $Bsplittemp1 = StringReplace ($sdlfileBsplit[1], '">', "TEMPSPLITTER", 1)
; $Bsplittemp2 = StringSplit ($Bsplittemp1, "TEMPSPLITTER", 1)

; $tagcontentraw = $Bsplittemp2[2]

_ArrayAdd ($tagcontents, $tagcontentraw)

Next

$numtagsx = Ubound($tagnums)
$numtags = $numtagsx - 1

; ==

; 3. Find tag IDs in segments, and create list file

$taglistfileopen = FileOpen (FileSaveDialog ("Create TagList file", @ScriptDir, "TXT (*.txt)|All (*.*)", 0, "taglistfile.txt"), 130) ; 130 = new file, UTF8 with BOM

FileWrite ($taglistfileopen, "Process all lines in this file?" & @TAB & "NO" & @CRLF & "Insert inside G tags?" & @TAB & "YES" & @CRLF)

For $l = 1 to $numtags

For $m = 1 to $numsegments

$gvalue = '<g id="' & $tagnums[$l] & '">'

If StringInStr ($segments[$m], $gvalue) Then

$tagcontents[$l] = StringRegExpReplace($tagcontents[$l], "\R", " ")
$segments[$m] = StringRegExpReplace($segments[$m], "\R", " ")

#cs
$tagcontents[$l] = StringReplace ($tagcontents[$l], @CRLF, " ")
$tagcontents[$l] = StringReplace ($tagcontents[$l], @LF, " ")
$tagcontents[$l] = StringReplace ($tagcontents[$l], @CR, " ")
$segments[$m] = StringReplace ($segments[$m], @CRLF, " ")
$segments[$m] = StringReplace ($segments[$m], @LF, " ")
$segments[$m] = StringReplace ($segments[$m], @CR, " ")
#ce

FileWrite ($taglistfileopen, $tagnums[$l] & @TAB & $tagcontents[$l] & @TAB & $segments[$m] & @CRLF)

EndIf

Next

Next
