
#include <Array.au3>

#cs

ID Populator v0.1
Samuel Murray, May 2015

SDLXLIFF files don't store formatting tags' actual content inside the segmented text, but instead refers to it by an ID that is defined in the head of the SDLXLIFF file.  This means that programs like Wordfast Pro or OmegaT that read SDLXLIFF files don't show the user what a tag means (it shows only the tag ID number).

The ID Populator scripts alter the SDXLIFF file so that tags' meanings are present inside the segments, so that if users process those SDXLIFF files in another tool, he/she may know what the tags actually mean.  The scripts give the option to clarify all tags, or only tags that the user selects (e.g. if the user is only interested in seeing which tags mean "highlight yellow", then he can set it so that only those tags are explained in the segments).

Script #1 creates a tag list file with all formatting tags in it.
The user then edits that list file to (a) remove all lines that he is not interested in or (b) mark up only the formatting that he is interested in.

Script #2 uses the tag list file to edit the SDLXLIFF file by adding the formatting to the segments, in curly brackets.

Script #3 is used in the end, to remove the added formatting information from the SDLXLIFF file again.

The tag list file has 3 columns, namely tag ID, tag content, and the segment in which it occurs.  Column 3 is not required to be present when running script #2.  Any line with fewer than 2 columns is not processed.

At the top of the tag list file are two YES/NO questions with two columns each:
* Question 1: Process all lines?
If the word "YES" is in the second column, then all lines in the tag list file is processed and the full text in column 2 of each tag is inserted into the SDLXLIFF file; if the word "NO" is in the second column, then only lines that contain {{ is processed, and only the text between {{ and }} is inserted into the SDLXLIFF file.
* Question 2: Put it inside G tag?
If "YES", then the script will change e.g. <g id="123"> to <g ttx="{{tag content}}" id="123">, which will cause Trados to complain when it loads the file, but it will load it anyway, and when you press Save in Trados, it quietly removes the inserted content.  If "NO", then the tag content is inserted into the SDXLIFF file directly after the relevant G tag, between {{ and }}.  This means that script #3 has to be used to remove it afterwards, otherwise it remains visible inside Trados and insert the final file also.

#ce

Global $sdlfileread

$taglistfileread = FileRead (FileOpen (FileOpenDialog ("Select Tag List file", @ScriptDir, "TXT (*.txt)|All (*.*)", 0, "taglistfile.txt"), 128))

$sdlfilediag = FileOpenDialog ("Select SDLXLIFF file", @ScriptDir, "SDLXLIFF (*.sdlxliff)|All (*.*)")

$sdlfileread = FileRead (FileOpen ($sdlfilediag, 256)) ; 256 = UTF8 without BOM, open for editing

If StringInStr ($sdlfileread, "{{") OR StringInStr ($sdlfileread, "}}") Then
MsgBox (0, "Error", "SDLXLIFF file already contains {{ or }}", 0)
Exit
EndIf

; 1. Read list file

$listfirstsplit = StringSplit ($taglistfileread, @CRLF, 1)

If StringInStr ($listfirstsplit[1], @TAB) Then
$firstlinesplit = StringSplit ($listfirstsplit[1], @TAB, 1)
If $firstlinesplit[2] = "YES" Then
$processall = 1
ElseIf $firstlinesplit[2] = "NO" Then
$processall = 0
Else
MsgBox (0, "Error", "Error: either YES or NO expected in line 1 of Tag List file", 0)
Exit
EndIf
Else
MsgBox (0, "Error", "Error: TAB character expected in line 1 of Tag List file", 0)
Exit
EndIf

If StringInStr ($listfirstsplit[2], @TAB) Then
$firstlinesplit = StringSplit ($listfirstsplit[2], @TAB, 1)
If $firstlinesplit[2] = "YES" Then
$insertintog = 1
ElseIf $firstlinesplit[2] = "NO" Then
$insertintog = 0
Else
MsgBox (0, "Error", "Error: either YES or NO expected in line 2 of Tag List file", 0)
Exit
EndIf
Else
MsgBox (0, "Error", "Error: TAB character expected in line 2 of Tag List file", 0)
Exit
EndIf

; The big for-next loop

For $i = 3 to $listfirstsplit[0]

If $processall = 0 Then
If NOT StringInStr ($listfirstsplit[$i], "{{") Then
ContinueLoop
EndIf
EndIf

If StringInStr ($listfirstsplit[$i], @TAB) Then
$listsecondsplit = StringSplit ($listfirstsplit[$i], @TAB, 1)
$gtag = '<g id="' & $listsecondsplit[1] & '">'
Else
ContinueLoop
EndIf

If $insertintog = 1 Then

$rawtagcontent = $listsecondsplit[2]
Local $a_rep[8][2] = [["&", "&amp;"], ["<", "&lt;"], [">", "&gt;"], ['"', "&quot;"], [@CRLF, " "], [@LF, " "], [@CR, " "], [@TAB, " "]]
For $p = 0 To Ubound($a_rep, 1) - 1
$cleantagcontent = StringRegExpReplace($rawtagcontent, $a_rep[$p][0], $a_rep[$p][1])
Next

ElseIf $insertintog = 0 Then

$rawtagcontent = $listsecondsplit[2]
Local $a_rep[7][2] = [["&", "&amp;"], ["<", "&lt;"], [">", "&gt;"], [@CRLF, " "], [@LF, " "], [@CR, " "], [@TAB, " "]]
For $p = 0 To Ubound($a_rep, 1) - 1
$cleantagcontent = StringRegExpReplace($rawtagcontent, $a_rep[$p][0], $a_rep[$p][1])
Next

EndIf


#cs
If $insertintog = 1 Then
$rawtagcontent = $listsecondsplit[2]
$cleantagcontent = StringReplace ($rawtagcontent, "&", "&amp;")
$cleantagcontent = StringReplace ($cleantagcontent, "<", "&lt;")
$cleantagcontent = StringReplace ($cleantagcontent, ">", "&gt;")
$cleantagcontent = StringReplace ($cleantagcontent, '"', "&quot;") ; this one has quot as well
$cleantagcontent = StringReplace ($cleantagcontent, @CRLF, " ")
$cleantagcontent = StringReplace ($cleantagcontent, @LF, " ")
$cleantagcontent = StringReplace ($cleantagcontent, @CR, " ")
$cleantagcontent = StringReplace ($cleantagcontent, @TAB, " ")
ElseIf $insertintog = 0 Then
$rawtagcontent = $listsecondsplit[2]
$cleantagcontent = StringReplace ($rawtagcontent, "&", "&amp;")
$cleantagcontent = StringReplace ($cleantagcontent, "<", "&lt;")
$cleantagcontent = StringReplace ($cleantagcontent, ">", "&gt;")
$cleantagcontent = StringReplace ($cleantagcontent, @CRLF, " ")
$cleantagcontent = StringReplace ($cleantagcontent, @LF, " ")
$cleantagcontent = StringReplace ($cleantagcontent, @CR, " ")
$cleantagcontent = StringReplace ($cleantagcontent, @TAB, " ")
EndIf
#ce

#cs
If $processall = 0 Then
$listfirstsplit[$i] = StringReplace ($listfirstsplit[$i], "{{", "{{")
$curlycount = @extended ; counts the number {{ in the line
EndIf
#ce

If $processall = 0 Then
$groupedtagcontent = ""
$curlysplitA = StringSplit ($cleantagcontent, "{{", 1)
For $j = 2 to $curlysplitA[0]
$curlysplitB = StringSplit ($curlysplitA[$j], "}}", 1)
$actualtagcontent = $curlysplitB[1]
$groupedtagcontent = $groupedtagcontent & "{{" & $actualtagcontent & "}}"
Next
ElseIf $processall = 1 Then
$groupedtagcontent = "{{" & $cleantagcontent & "}}"
EndIf

If $insertintog = 0 Then
$replacegtag = '<g id="' & $listsecondsplit[1] & '">' & $groupedtagcontent
ElseIf $insertintog = 1 Then
$replacegtag = '<g id="' & $listsecondsplit[1] & '" ttx="' & $groupedtagcontent & '">'
EndIf

; so now we have:
; $gtag
; $replacegtag

; 2. In SDLXLIFF file, replace

$sdlfileread = StringReplace ($sdlfileread, $gtag, $replacegtag)

Next

$sdlwritefile = $sdlfilediag & "_popped.sdlxliff"

$sdlwritefileopen = FileOpen ($sdlwritefile, 258)

FileWrite ($sdlwritefileopen, $sdlfileread)

